<?php

namespace Telnyx;

/**
 * Class MobileOperatorNetwork
 *
 * @package Telnyx
 */
class MobileOperatorNetwork extends ApiResource
{
	
    const OBJECT_NAME = "mobile_operator_network";

    use ApiOperations\All;
    
}
