<?php
/**
 * @copyright 2021 Double Bastion LLC <www.doublebastion.com>
 *
 * @author Double Bastion LLC
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

declare(strict_types=1);

namespace OCA\SMSRelentless\Migration;

use Closure;
use OCP\DB\Types;
use OCP\DB\ISchemaWrapper;
use OCP\IDBConnection;
use OCP\Migration\IOutput;
use OCP\Migration\SimpleMigrationStep;


class Version116Date20230508212745 extends SimpleMigrationStep {

        /** @var IDBConnection */
        private $connection;

        /**
         * @param IDBConnection $connection
         */
        public function __construct(IDBConnection $connection) {
                $this->connection = $connection;
        }

	/**
	 * @param IOutput $output
	 * @param Closure $schemaClosure The `\Closure` returns a `ISchemaWrapper`
	 * @param array $options
	 * @return null|ISchemaWrapper
	 */
	public function changeSchema(IOutput $output, Closure $schemaClosure, array $options): ?ISchemaWrapper {
		/** @var ISchemaWrapper $schema */
		$schema = $schemaClosure();

		if (!$schema->hasTable('sms_relent_settings')) {
			$table = $schema->createTable('sms_relent_settings');
			$table->addColumn('id', Types::BIGINT, [
				'autoincrement' => true,
				'notnull' => true,
                                'length' => 11,
                                'unsigned' => true,
			]);
			$table->addColumn('user_id', Types::STRING, [
				'notnull' => true,
				'length' => 128,
			]);
			$table->addColumn('telapi_key', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('tel_pub_key', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('telapi_url_rec', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('telapi_url', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('messaging_profile_id', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('nexapi_key', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('nexapi_secret', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('nexapi_url_rec', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('nexapi_url', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('twilapi_key', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('twilapi_secret', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('twilapi_url_rec', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('twilapi_url', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('flowapi_key', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('flowapi_secret', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('flowapi_url_rec', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('flowapi_url', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('tel_sender_name', Types::STRING, [
				'notnull' => false,
				'length' => 48,
				'default' => '',
			]);
			$table->addColumn('nex_sender_name', Types::STRING, [
				'notnull' => false,
				'length' => 48,
				'default' => '',
			]);
			$table->addColumn('twil_sender_name', Types::STRING, [
				'notnull' => false,
				'length' => 48,
				'default' => '',
			]);
			$table->addColumn('flow_sender_name', Types::STRING, [
				'notnull' => false,
				'length' => 48,
				'default' => '',
			]);
			$table->addColumn('messagesperpage', Types::INTEGER, [
				'notnull' => false,
				'length' => 11,
                                'unsigned' => true,
			]);
			$table->addColumn('get_notify', Types::SMALLINT, [
				'notnull' => false,
				'length' => 1,
			]);
			$table->addColumn('notification_email', Types::STRING, [
				'notnull' => false,
				'length' => 512,
				'default' => '',
			]);
			$table->addColumn('getsmsinemail', Types::SMALLINT, [
				'notnull' => false,
				'length' => 1,
			]);
			$table->addColumn('show_all_messages', Types::SMALLINT, [
				'notnull' => false,
				'length' => 1,
			]);
			$table->addColumn('show_display_names', Types::SMALLINT, [
				'notnull' => false,
				'length' => 1,
			]);
                        $table->setPrimaryKey(['id']);
                        $table->addUniqueIndex(['id']);

		} else {

                        $table = $schema->getTable('sms_relent_settings');

			$table->addColumn('show_all_messages', Types::SMALLINT, [
				'notnull' => false,
				'length' => 1,
			]);
			$table->addColumn('show_display_names', Types::SMALLINT, [
				'notnull' => false,
				'length' => 1,
			]);
                }


		if (!$schema->hasTable('sms_relent_received')) {
			$table = $schema->createTable('sms_relent_received');
			$table->addColumn('id', Types::BIGINT, [
				'autoincrement' => true,
				'notnull' => true,
                                'length' => 11,
                                'unsigned' => true,
			]);
			$table->addColumn('user_id', Types::STRING, [
				'notnull' => true,
				'length' => 64,
			]);
			$table->addColumn('message_id', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('date', Types::DATETIME, [
				'notnull' => true,
			]);
			$table->addColumn('from', Types::STRING, [
				'notnull' => true,
				'length' => 128,
			]);
			$table->addColumn('to', Types::STRING, [
				'notnull' => true,
				'length' => 128,
			]);
			$table->addColumn('message', Types::TEXT, [
				'notnull' => false,
				'default' => '',
			]);
			$table->addColumn('author_displayname', Types::STRING, [
				'notnull' => true,
				'length' => 255,
			]);
			$table->addColumn('internal_sender', Types::SMALLINT, [
				'notnull' => false,
				'length' => 1,
			]);
                        $table->setPrimaryKey(['id']);
                        $table->addUniqueIndex(['id']);

		} else {

                        $table = $schema->getTable('sms_relent_received');

			$table->addColumn('author_displayname', Types::STRING, [
				'notnull' => true,
				'length' => 255,
			]);
			$table->addColumn('internal_sender', Types::SMALLINT, [
				'notnull' => false,
				'length' => 1,
			]);
                }


		if (!$schema->hasTable('sms_relent_sent')) {
			$table = $schema->createTable('sms_relent_sent');
			$table->addColumn('id', Types::BIGINT, [
				'autoincrement' => true,
				'notnull' => true,
                                'length' => 11,
                                'unsigned' => true,
			]);
			$table->addColumn('user_id', Types::STRING, [
				'notnull' => true,
				'length' => 64,
			]);
			$table->addColumn('message_id', Types::STRING, [
				'notnull' => true,
				'length' => 512,
			]);
			$table->addColumn('date', Types::DATETIME, [
				'notnull' => true,
			]);
			$table->addColumn('from', Types::STRING, [
				'notnull' => true,
				'length' => 128,
			]);
			$table->addColumn('to', Types::STRING, [
				'notnull' => true,
				'length' => 128,
			]);
			$table->addColumn('network ', Types::STRING, [
				'notnull' => false,
				'length' => 64,
				'default' => '',
			]);
			$table->addColumn('price', Types::STRING, [
				'notnull' => false,
				'length' => 64,
				'default' => '',
			]);
			$table->addColumn('status', Types::STRING, [
				'notnull' => false,
				'length' => 512,
				'default' => '',
			]);
			$table->addColumn('deliveryreceipt', Types::STRING, [
				'notnull' => false,
				'length' => 64,
				'default' => '',
			]);
			$table->addColumn('message', Types::TEXT, [
				'notnull' => false,
                                'default' => '',
			]);
			$table->addColumn('author_displayname', Types::STRING, [
				'notnull' => true,
				'length' => 255,
			]);
                        $table->setPrimaryKey(['id']);
                        $table->addUniqueIndex(['id']);

		} else {

                        $table = $schema->getTable('sms_relent_sent');

			$table->addColumn('author_displayname', Types::STRING, [
				'notnull' => true,
				'length' => 255,
			]);
                }


		if (!$schema->hasTable('sms_relent_autorply')) {
			$table = $schema->createTable('sms_relent_autorply');
			$table->addColumn('id', Types::BIGINT, [
				'autoincrement' => true,
				'notnull' => true,
                                'length' => 11,
                                'unsigned' => true,
			]);
			$table->addColumn('user_id', Types::STRING, [
				'notnull' => true,
				'length' => 128,
			]);
			$table->addColumn('saved_by_dsplname', Types::STRING, [
				'notnull' => true,
				'length' => 255,
			]);
			$table->addColumn('phone_number', Types::STRING, [
				'notnull' => true,
				'length' => 128,
			]);
			$table->addColumn('days_of_week', Types::STRING, [
				'notnull' => false,
				'length' => 64,
                                'default' => '',
			]);
			$table->addColumn('daily_start', Types::STRING, [
				'notnull' => false,
				'length' => 8,
                                'default' => '',
			]);
			$table->addColumn('daily_end', Types::STRING, [
				'notnull' => false,
				'length' => 8,
                                'default' => '',
			]);
			$table->addColumn('vacation_start', Types::DATETIME, [
				'notnull' => false,
			]);
			$table->addColumn('vacation_end', Types::DATETIME, [
				'notnull' => false,
			]);
			$table->addColumn('message_text', Types::TEXT, [
				'notnull' => false,
                                'default' => '',
			]);
                        $table->setPrimaryKey(['id']);
                        $table->addUniqueIndex(['id']);

		}


		if (!$schema->hasTable('sms_relent_restrict')) {
			$table = $schema->createTable('sms_relent_restrict');
			$table->addColumn('id', Types::BIGINT, [
				'autoincrement' => true,
				'notnull' => true,
                                'length' => 11,
                                'unsigned' => true,
			]);
			$table->addColumn('user_id', Types::STRING, [
				'notnull' => true,
				'length' => 128,
			]);
			$table->addColumn('saved_by_dsplname', Types::STRING, [
				'notnull' => true,
				'length' => 255,
			]);
			$table->addColumn('phone_number', Types::STRING, [
				'notnull' => true,
				'length' => 128,
			]);
			$table->addColumn('groups', Types::STRING, [
				'notnull' => true,
				'length' => 2048,
			]);
			$table->addColumn('users', Types::STRING, [
				'notnull' => true,
				'length' => 2048,
			]);
                        $table->setPrimaryKey(['id']);
                        $table->addUniqueIndex(['id']);

		}

		return $schema;
	}

        public function postSchemaChange(IOutput $output, Closure $schemaClosure, array $options): ?ISchemaWrapper {

                // Enter the Nextcloud display name in the 'author_displayname' column of the 'sms_relent_sent' table, for messages that were sent before the upgrade
                $getrecfromnb = $this->connection->prepare('SELECT `id`, `user_id` FROM `*PREFIX*sms_relent_sent`');
	        $getrecfromnbres = $getrecfromnb->execute();

                $recdatauids = [];
                $idsent = [];
                while ($rowfetched = $getrecfromnbres->fetch()) {
                       $recdatauids[] = $rowfetched['user_id'];
                       $idsent[] = $rowfetched['id'];
                }
                $getrecfromnbres->closeCursor();

                if ($recdatauids) {

                    $recuids = array_values(array_unique($recdatauids));
                    $acdatausrdn = [];

                    foreach ($recuids as $dnkey => $dnvalue) {
		             $getacdatadn = $this->connection->prepare('SELECT `uid`, `name`, `value` FROM `*PREFIX*accounts_data` WHERE `uid` = ? AND `name` = ?');
			     $getacdatadnres = $getacdatadn->execute([$dnvalue, 'displayname']);

			     while ($acusrdnfetched = $getacdatadnres->fetch()) {
				    $acdatausrdn[$dnvalue] = $acusrdnfetched['value'];
			     }
			     $getacdatadnres->closeCursor();
                    }

                    foreach ($recdatauids as $pdpkey => $pdpvalue) {

			     $updaterecms = $this->connection->prepare('UPDATE `*PREFIX*sms_relent_sent` SET `author_displayname` = ? WHERE `user_id` = ? AND `id` = ?');
			     $updaterecmsres = $updaterecms->execute([$acdatausrdn[$pdpvalue], $pdpvalue, $idsent[$pdpkey]]);
			     $updaterecmsres->closeCursor();
                    }
                }


                // Enter the Nextcloud display name in the 'author_displayname' column of the 'sms_relent_received' table, for messages that were received before the upgrade
                $getrecfromnb = $this->connection->prepare('SELECT `id`, `date`, `from` FROM `*PREFIX*sms_relent_received`');
	        $getrecfromnbres = $getrecfromnb->execute();

                $recdatanmbrs = [];
                $idsrc = [];
                $daterec = [];
                while ($rowfetched = $getrecfromnbres->fetch()) {
                       $recdatanmbrs[] = $rowfetched['from'];
                       $idsrc[] = $rowfetched['id'];
                       $daterecarr = explode(":", $rowfetched['date']);
                       $daterec[] = $daterecarr[0];
                }
                $getrecfromnbres->closeCursor();

                if ($recdatanmbrs) {

                    foreach ($recdatanmbrs as $rckey => $rcvalue) {

			     $authorDisplayNm = '';
			     $internalSender = 0;

			     // Search for the associated Display Name in the previous messages sent from the same phone number, in the 'sms_relent_sent' table
			     $recmsgfromprc = '%' . $rcvalue;
                             $recdataprc = $daterec[$rckey] . '%';
			     $getdspnmst = $this->connection->prepare('SELECT `date`, `from`, `author_displayname` FROM `*PREFIX*sms_relent_sent` WHERE `from` LIKE ? AND `date` LIKE ?');
			     $getdspnmstres = $getdspnmst->execute([$recmsgfromprc, $recdataprc]);

			     $dispnamearr = [];
			     while ($dspnmstfetched = $getdspnmstres->fetch()) {
				    $dispnamearr[] = $dspnmstfetched['author_displayname'];
			     }
			     $getdspnmstres->closeCursor();

			     if ($dispnamearr) {
				 $descdspnmarr = array_reverse($dispnamearr);
				 $authorDisplayNm = $descdspnmarr[0];
				 $internalSender = 1;
			     } else { $authorDisplayNm = ''; $internalSender = 0; }

                             $pdpkeyprc = '%' . $rcvalue;
			     $updaterecms = $this->connection->prepare('UPDATE `*PREFIX*sms_relent_received` SET `author_displayname` = ?, `internal_sender` = ?
                                                                        WHERE `from` LIKE ? AND `id` = ?');
			     $updaterecmsres = $updaterecms->execute([$authorDisplayNm, $internalSender, $pdpkeyprc, $idsrc[$rckey]]);
			     $updaterecmsres->closeCursor();
                    }
                }

                return null;
        }

}
